@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "0.5.0"
+#define BARESIP_VERSION "0.5.4"
 
 
 #ifndef NET_MAX_NS
@@ -55,7 +55,25 @@ struct list *account_vidcodecl(const struct account *acc);
 struct sip_addr *account_laddr(const struct account *acc);
 uint32_t account_regint(const struct account *acc);
 uint32_t account_pubint(const struct account *acc);
+uint32_t account_ptime(const struct account *acc);
 enum answermode account_answermode(const struct account *acc);
+const char *account_auth_user(const struct account *acc);
+const char *account_outbound(const struct account *acc, unsigned ix);
+const char *account_stun_user(const struct account *acc);
+const char *account_stun_pass(const struct account *acc);
+const char *account_stun_host(const struct account *acc);
+
+
+/*
+ * Audio-level
+ */
+
+
+#define AULEVEL_MIN  (-96.0)
+#define AULEVEL_MAX    (0.0)
+
+
+double aulevel_calc_dbov(const int16_t *sampv, size_t sampc);
 
 
 /*
@@ -182,6 +200,7 @@ struct config_audio {
 	uint32_t channels_src;  /**< Opt. channels for source       */
 	bool src_first;         /**< Audio source opened first      */
 	enum audio_mode txmode; /**< Audio transmit mode            */
+	bool level;             /**< Enable audio level indication  */
 };
 
 #ifdef USE_VIDEO
@@ -303,9 +322,14 @@ struct media_ctx {
 typedef void (message_recv_h)(const struct pl *peer, const struct pl *ctype,
 			      struct mbuf *body, void *arg);
 
-int  message_init(message_recv_h *recvh, void *arg);
-void message_close(void);
-int  message_send(struct ua *ua, const char *peer, const char *msg);
+struct message;
+struct message_lsnr;
+
+int  message_init(struct message **messagep);
+int  message_listen(struct message_lsnr **lsnrp, struct message *message,
+		    message_recv_h *h, void *arg);
+int  message_send(struct ua *ua, const char *peer, const char *msg,
+		  sip_resp_h *resph, void *arg);
 
 
 /*
@@ -330,10 +354,11 @@ typedef int  (ausrc_alloc_h)(struct ausrc_st **stp, const struct ausrc *ausrc,
 			     struct ausrc_prm *prm, const char *device,
 			     ausrc_read_h *rh, ausrc_error_h *errh, void *arg);
 
-int ausrc_register(struct ausrc **asp, const char *name,
+int ausrc_register(struct ausrc **asp, struct list *ausrcl, const char *name,
 		   ausrc_alloc_h *alloch);
-const struct ausrc *ausrc_find(const char *name);
-int ausrc_alloc(struct ausrc_st **stp, struct media_ctx **ctx,
+const struct ausrc *ausrc_find(const struct list *ausrcl, const char *name);
+int ausrc_alloc(struct ausrc_st **stp, struct list *ausrcl,
+		struct media_ctx **ctx,
 		const char *name,
 		struct ausrc_prm *prm, const char *device,
 		ausrc_read_h *rh, ausrc_error_h *errh, void *arg);
@@ -359,10 +384,11 @@ typedef int  (auplay_alloc_h)(struct auplay_st **stp, const struct auplay *ap,
 			      struct auplay_prm *prm, const char *device,
 			      auplay_write_h *wh, void *arg);
 
-int auplay_register(struct auplay **pp, const char *name,
-		    auplay_alloc_h *alloch);
-const struct auplay *auplay_find(const char *name);
-int auplay_alloc(struct auplay_st **stp, const char *name,
+int auplay_register(struct auplay **pp, struct list *auplayl,
+		    const char *name, auplay_alloc_h *alloch);
+const struct auplay *auplay_find(const struct list *auplayl, const char *name);
+int auplay_alloc(struct auplay_st **stp, struct list *auplayl,
+		 const char *name,
 		 struct auplay_prm *prm, const char *device,
 		 auplay_write_h *wh, void *arg);
 
@@ -410,9 +436,8 @@ struct aufilt {
 	aufilt_decode_h *dech;
 };
 
-void aufilt_register(struct aufilt *af);
+void aufilt_register(struct list *aufiltl, struct aufilt *af);
 void aufilt_unregister(struct aufilt *af);
-struct list *aufilt_list(void);
 
 
 /*
@@ -473,9 +498,9 @@ struct menc {
 	menc_media_h *mediah;
 };
 
-void menc_register(struct menc *menc);
+void menc_register(struct list *mencl, struct menc *menc);
 void menc_unregister(struct menc *menc);
-const struct menc *menc_find(const char *id);
+const struct menc *menc_find(const struct list *mencl, const char *id);
 
 
 /*
@@ -563,6 +588,7 @@ int  ua_connect(struct ua *ua, struct call **callp,
 void ua_hangup(struct ua *ua, struct call *call,
 	       uint16_t scode, const char *reason);
 int  ua_answer(struct ua *ua, struct call *call);
+int  ua_progress(struct ua *ua, struct call *call);
 int  ua_hold_answer(struct ua *ua, struct call *call);
 int  ua_options_send(struct ua *ua, const char *uri,
 		     options_resp_h *resph, void *arg);
@@ -674,14 +700,10 @@ struct cmd {
 };
 
 struct cmd_ctx;
-
-struct commands {
-	struct list cmdl;        /**< List of command blocks (struct cmds) */
-};
+struct commands;
 
 
-int  cmd_init(struct commands *commands);
-void cmd_close(struct commands *commands);
+int  cmd_init(struct commands **commandsp);
 int  cmd_register(struct commands *commands,
 		  const struct cmd *cmdv, size_t cmdc);
 void cmd_unregister(struct commands *commands, const struct cmd *cmdv);
@@ -722,11 +744,11 @@ typedef int  (vidsrc_alloc_h)(struct vidsrc_st **vsp, const struct vidsrc *vs,
 typedef void (vidsrc_update_h)(struct vidsrc_st *st, struct vidsrc_prm *prm,
 			       const char *dev);
 
-int vidsrc_register(struct vidsrc **vp, const char *name,
+int vidsrc_register(struct vidsrc **vp, struct list *vidsrcl, const char *name,
 		    vidsrc_alloc_h *alloch, vidsrc_update_h *updateh);
-const struct vidsrc *vidsrc_find(const char *name);
-struct list *vidsrc_list(void);
-int vidsrc_alloc(struct vidsrc_st **stp, const char *name,
+const struct vidsrc *vidsrc_find(const struct list *vidsrcl, const char *name);
+int vidsrc_alloc(struct vidsrc_st **stp, struct list *vidsrcl,
+		 const char *name,
 		 struct media_ctx **ctx, struct vidsrc_prm *prm,
 		 const struct vidsz *size, const char *fmt, const char *dev,
 		 vidsrc_frame_h *frameh, vidsrc_error_h *errorh, void *arg);
@@ -756,15 +778,16 @@ typedef int  (vidisp_disp_h)(struct vidisp_st *st, const char *title,
 			     const struct vidframe *frame);
 typedef void (vidisp_hide_h)(struct vidisp_st *st);
 
-int vidisp_register(struct vidisp **vp, const char *name,
+int vidisp_register(struct vidisp **vp, struct list *vidispl, const char *name,
 		    vidisp_alloc_h *alloch, vidisp_update_h *updateh,
 		    vidisp_disp_h *disph, vidisp_hide_h *hideh);
-int vidisp_alloc(struct vidisp_st **stp, const char *name,
+int vidisp_alloc(struct vidisp_st **stp, struct list *vidispl,
+		 const char *name,
 		 struct vidisp_prm *prm, const char *dev,
 		 vidisp_resize_h *resizeh, void *arg);
 int vidisp_display(struct vidisp_st *st, const char *title,
 		   const struct vidframe *frame);
-const struct vidisp *vidisp_find(const char *name);
+const struct vidisp *vidisp_find(const struct list *vidispl, const char *name);
 
 
 /*
@@ -810,11 +833,11 @@ struct aucodec {
 	sdp_fmtp_cmp_h *fmtp_cmph;
 };
 
-void aucodec_register(struct aucodec *ac);
+void aucodec_register(struct list *aucodecl, struct aucodec *ac);
 void aucodec_unregister(struct aucodec *ac);
-const struct aucodec *aucodec_find(const char *name, uint32_t srate,
+const struct aucodec *aucodec_find(const struct list *aucodecl,
+				   const char *name, uint32_t srate,
 				   uint8_t ch);
-struct list *aucodec_list(void);
 
 
 /*
@@ -863,12 +886,14 @@ struct vidcodec {
 	sdp_fmtp_cmp_h *fmtp_cmph;
 };
 
-void vidcodec_register(struct vidcodec *vc);
+void vidcodec_register(struct list *vidcodecl, struct vidcodec *vc);
 void vidcodec_unregister(struct vidcodec *vc);
-const struct vidcodec *vidcodec_find(const char *name, const char *variant);
-const struct vidcodec *vidcodec_find_encoder(const char *name);
-const struct vidcodec *vidcodec_find_decoder(const char *name);
-struct list *vidcodec_list(void);
+const struct vidcodec *vidcodec_find(const struct list *vidcodecl,
+				     const char *name, const char *variant);
+const struct vidcodec *vidcodec_find_encoder(const struct list *vidcodecl,
+					     const char *name);
+const struct vidcodec *vidcodec_find_decoder(const struct list *vidcodecl,
+					     const char *name);
 
 
 /*
@@ -907,9 +932,8 @@ struct vidfilt {
 	vidfilt_decode_h *dech;
 };
 
-void vidfilt_register(struct vidfilt *vf);
+void vidfilt_register(struct list *vidfiltl, struct vidfilt *vf);
 void vidfilt_unregister(struct vidfilt *vf);
-struct list *vidfilt_list(void);
 int vidfilt_enc_append(struct list *filtl, void **ctx,
 		       const struct vidfilt *vf);
 int vidfilt_dec_append(struct list *filtl, void **ctx,
@@ -928,6 +952,7 @@ void audio_set_devicename(struct audio *a, const char *src, const char *play);
 int  audio_set_source(struct audio *au, const char *mod, const char *device);
 int  audio_set_player(struct audio *au, const char *mod, const char *device);
 void audio_encoder_cycle(struct audio *audio);
+int  audio_level_get(const struct audio *au, double *level);
 int  audio_debug(struct re_printf *pf, const struct audio *a);
 
 
@@ -971,7 +996,8 @@ typedef int (mnat_media_h)(struct mnat_media **mp, struct mnat_sess *sess,
 
 typedef int (mnat_update_h)(struct mnat_sess *sess);
 
-int mnat_register(struct mnat **mnatp, const char *id, const char *ftag,
+int mnat_register(struct mnat **mnatp, struct list *mnatl,
+		  const char *id, const char *ftag,
 		  mnat_sess_h *sessh, mnat_media_h *mediah,
 		  mnat_update_h *updateh);
 
@@ -1111,6 +1137,17 @@ struct network *baresip_network(void);
 struct contacts *baresip_contacts(void);
 struct commands *baresip_commands(void);
 struct player *baresip_player(void);
+struct message *baresip_message(void);
+struct list   *baresip_mnatl(void);
+struct list   *baresip_mencl(void);
+struct list   *baresip_aucodecl(void);
+struct list   *baresip_ausrcl(void);
+struct list   *baresip_auplayl(void);
+struct list   *baresip_aufiltl(void);
+struct list   *baresip_vidcodecl(void);
+struct list   *baresip_vidsrcl(void);
+struct list   *baresip_vidispl(void);
+struct list   *baresip_vidfiltl(void);
 
 
 #ifdef __cplusplus
