@@ -32,7 +32,8 @@
  * Config options:
  *
  \verbatim
-      avcodec_h264enc  <NAME>  ; e.g. nvenc_h264, h264_videotoolbox
+      avcodec_h264enc  <NAME>  ; e.g. h264_nvenc, h264_videotoolbox
+      avcodec_h264dec  <NAME>  ; e.g. h264_cuvid, h264_vda, h264_qsv
  \endverbatim
  *
  * References:
@@ -47,7 +48,8 @@
 
 
 const uint8_t h264_level_idc = 0x0c;
-AVCodec *avcodec_h264enc;             /* optinal; specified H.264 encoder */
+AVCodec *avcodec_h264enc;             /* optional; specified H.264 encoder */
+AVCodec *avcodec_h264dec;             /* optional; specified H.264 decoder */
 
 
 int avcodec_resolve_codecid(const char *s)
@@ -181,7 +183,9 @@ static struct vidcodec mpg4 = {
 
 static int module_init(void)
 {
+	struct list *vidcodecl = baresip_vidcodecl();
 	char h264enc[64];
+	char h264dec[64];
 
 #ifdef USE_X264
 	debug("avcodec: x264 build %d\n", X264_BUILD);
@@ -195,14 +199,29 @@ static int module_init(void)
 
 	avcodec_register_all();
 
-	if (avcodec_find_decoder(AV_CODEC_ID_H264))
-		vidcodec_register(&h264);
+	if (0 == conf_get_str(conf_cur(), "avcodec_h264dec",
+			      h264dec, sizeof(h264dec))) {
+
+		info("avcodec: using h264 decoder by name (%s)\n", h264dec);
+
+		avcodec_h264dec = avcodec_find_decoder_by_name(h264dec);
+		if (!avcodec_h264dec) {
+			warning("avcodec: h264 decoder not found (%s)\n",
+				h264dec);
+			return ENOENT;
+		}
+		vidcodec_register(vidcodecl, &h264);
+	}
+	else {
+		if (avcodec_find_decoder(AV_CODEC_ID_H264))
+			vidcodec_register(vidcodecl, &h264);
+	}
 
 	if (avcodec_find_decoder(AV_CODEC_ID_H263))
-		vidcodec_register(&h263);
+		vidcodec_register(vidcodecl, &h263);
 
 	if (avcodec_find_decoder(AV_CODEC_ID_MPEG4))
-		vidcodec_register(&mpg4);
+		vidcodec_register(vidcodecl, &mpg4);
 
 	if (0 == conf_get_str(conf_cur(), "avcodec_h264enc",
 			      h264enc, sizeof(h264enc))) {
