@@ -518,7 +518,7 @@ static int src_alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 	if (!stp || !size || !frameh)
 		return EINVAL;
 
-	if (str_isset(dev))
+	if (!str_isset(dev))
 		dev = "/dev/video0";
 
 	debug("v4l2_codec: video-source alloc (device=%s)\n", dev);
@@ -562,8 +562,9 @@ static int module_init(void)
 {
 	info("v4l2_codec inited\n");
 
-	vidcodec_register(&h264);
-	return vidsrc_register(&vidsrc, "v4l2_codec", src_alloc, NULL);
+	vidcodec_register(baresip_vidcodecl(), &h264);
+	return vidsrc_register(&vidsrc, baresip_vidsrcl(),
+			       "v4l2_codec", src_alloc, NULL);
 }
 
 
