@@ -233,6 +233,7 @@ static void vidloop_destructor(void *arg)
 
 static int enable_codec(struct video_loop *vl, const char *name)
 {
+	struct list *vidcodecl = baresip_vidcodecl();
 	struct videnc_param prm;
 	int err;
 
@@ -243,7 +244,7 @@ static int enable_codec(struct video_loop *vl, const char *name)
 
 	/* Use the first video codec */
 
-	vl->vc_enc = vidcodec_find_encoder(name);
+	vl->vc_enc = vidcodec_find_encoder(vidcodecl, name);
 	if (!vl->vc_enc) {
 		warning("vidloop: could not find encoder (%s)\n", name);
 		return ENOENT;
@@ -252,7 +253,7 @@ static int enable_codec(struct video_loop *vl, const char *name)
 	info("vidloop: enabled encoder %s (%u fps, %u bit/s)\n",
 	     vl->vc_enc->name, prm.fps, prm.bitrate);
 
-	vl->vc_dec = vidcodec_find_decoder(name);
+	vl->vc_dec = vidcodec_find_decoder(vidcodecl, name);
 	if (!vl->vc_dec) {
 		warning("vidloop: could not find decoder (%s)\n", name);
 		return ENOENT;
@@ -341,7 +342,8 @@ static int vsrc_reopen(struct video_loop *vl, const struct vidsz *sz)
 	prm.fps    = vl->cfg.fps;
 
 	vl->vsrc = mem_deref(vl->vsrc);
-	err = vidsrc_alloc(&vl->vsrc, vl->cfg.src_mod, NULL, &prm, sz,
+	err = vidsrc_alloc(&vl->vsrc, baresip_vidsrcl(),
+			   vl->cfg.src_mod, NULL, &prm, sz,
 			   NULL, vl->cfg.src_dev, vidsrc_frame_handler,
 			   NULL, vl);
 	if (err) {
@@ -372,7 +374,7 @@ static int video_loop_alloc(struct video_loop **vlp, const struct vidsz *size)
 	tmr_init(&vl->tmr_bw);
 
 	/* Video filters */
-	for (le = list_head(vidfilt_list()); le; le = le->next) {
+	for (le = list_head(baresip_vidfiltl()); le; le = le->next) {
 		struct vidfilt *vf = le->data;
 		void *ctx = NULL;
 
@@ -392,7 +394,8 @@ static int video_loop_alloc(struct video_loop **vlp, const struct vidsz *size)
 	info("vidloop: open video display (%s.%s)\n",
 	     vl->cfg.disp_mod, vl->cfg.disp_dev);
 
-	err = vidisp_alloc(&vl->vidisp, vl->cfg.disp_mod, NULL,
+	err = vidisp_alloc(&vl->vidisp, baresip_vidispl(),
+			   vl->cfg.disp_mod, NULL,
 			   vl->cfg.disp_dev, NULL, vl);
 	if (err) {
 		warning("vidloop: video display failed: %m\n", err);
