@@ -20,7 +20,7 @@
  *     Briefly tested with Twinkle 1.4.2 and Jitsi 2.2.4603.9615
  *
  *     This module is using ZRTP implementation in Freeswitch
- *     https://github.com/traviscross/libzrtp
+ *     https://github.com/juha-h/libzrtp
  *
  * Thanks:
  *
@@ -47,6 +47,20 @@ struct menc_media {
 
 static zrtp_global_t *zrtp_global;
 static zrtp_config_t zrtp_config;
+static zrtp_zid_t zid;
+
+
+static inline bool is_rtp_or_rtcp(const struct mbuf *mb)
+{
+	uint8_t b;
+
+	if (mbuf_get_left(mb) < 1)
+		return false;
+
+	b = mbuf_buf(mb)[0];
+
+	return 127 < b && b < 192;
+}
 
 
 static void session_destructor(void *arg)
@@ -80,9 +94,19 @@ static bool udp_helper_send(int *err, struct sa *dst,
 
 	length = (unsigned int)mbuf_get_left(mb);
 
+	/* only RTP packets should be processed */
+	if (!is_rtp_or_rtcp(mb))
+		return false;
+
 	s = zrtp_process_rtp(st->zrtp_stream, (char *)mbuf_buf(mb), &length);
 	if (s != zrtp_status_ok) {
-		warning("zrtp: zrtp_process_rtp failed (status = %d)\n", s);
+
+		if (s == zrtp_status_drop)
+			return true;
+
+		warning("zrtp: send: zrtp_process_rtp failed"
+			" (status = %d '%s')\n",
+			s, zrtp_log_status2str(s));
 		return false;
 	}
 
@@ -114,7 +138,8 @@ static bool udp_helper_recv(struct sa *src, struct mbuf *mb, void *arg)
 		if (s == zrtp_status_drop)
 			return true;
 
-		warning("zrtp: zrtp_process_srtp: %d\n", s);
+		warning("zrtp: recv: zrtp_process_srtp: %d '%s'\n",
+			s, zrtp_log_status2str(s));
 		return false;
 	}
 
@@ -141,7 +166,7 @@ static int session_alloc(struct menc_sess **sessp, struct sdp_session *sdp,
 	if (!st)
 		return ENOMEM;
 
-	s = zrtp_session_init(zrtp_global, NULL,
+	s = zrtp_session_init(zrtp_global, NULL, zid,
 			      ZRTP_SIGNALING_ROLE_UNKNOWN, &st->zrtp_session);
 	if (s != zrtp_status_ok) {
 		warning("zrtp: zrtp_session_init failed (status = %d)\n", s);
@@ -260,10 +285,12 @@ static void on_zrtp_secure(zrtp_stream_t *stream)
 	zrtp_session_get(sess->zrtp_session, &sess_info);
 	if (!sess_info.sas_is_verified && sess_info.sas_is_ready) {
 		info("zrtp: verify SAS <%s> <%s> for remote peer %w"
-		     " (press 'Z' <ZID> to verify)\n",
+		     " (type /zrtp %w to verify)\n",
 		     sess_info.sas1.buffer,
 		     sess_info.sas2.buffer,
 		     sess_info.peer_zid.buffer,
+		     (size_t)sess_info.peer_zid.length,
+		     sess_info.peer_zid.buffer,
 		     (size_t)sess_info.peer_zid.length);
 	}
 }
@@ -282,8 +309,12 @@ static int verify_sas(struct re_printf *pf, void *arg)
 	if (str_isset(carg->prm)) {
 		char rzid[ZRTP_STRING16] = "";
 		zrtp_status_t s;
+		zrtp_string16_t local_zid = ZSTR_INIT_EMPTY(local_zid);
 		zrtp_string16_t remote_zid = ZSTR_INIT_EMPTY(remote_zid);
 
+		zrtp_zstrncpyc(ZSTR_GV(local_zid), (const char*)zid,
+			       sizeof(zrtp_zid_t));
+
 		if (str_len(carg->prm) != 24) {
 			warning("zrtp: invalid remote ZID (%s)\n", carg->prm);
 			return EINVAL;
@@ -294,13 +325,12 @@ static int verify_sas(struct re_printf *pf, void *arg)
 		zrtp_zstrncpyc(ZSTR_GV(remote_zid), (const char*)rzid,
 			       sizeof(zrtp_zid_t));
 
-		s = zrtp_cache_set_verified(zrtp_global->cache,
-					    ZSTR_GV(remote_zid),
-					    true);
+		s = zrtp_verified_set(zrtp_global, &local_zid, &remote_zid,
+				      true);
 		if (s == zrtp_status_ok)
 			info("zrtp: SAS for peer %s verified\n", carg->prm);
 		else {
-			warning("zrtp: zrtp_cache_set_verified"
+			warning("zrtp: zrtp_verified_set"
 				" failed (status = %d)\n", s);
 			return EINVAL;
 		}
@@ -310,8 +340,24 @@ static int verify_sas(struct re_printf *pf, void *arg)
 }
 
 
+static void zrtp_log(int level, char *data, int len, int offset)
+{
+	(void)offset;
+
+	if (level == 1) {
+		error("%b\n", data, len);
+	}
+	else if (level == 2) {
+		info("%b\n", data, len);
+	}
+	else {
+		debug("%b\n", data, len);
+	}
+}
+
+
 static const struct cmd cmdv[] = {
-	{"zrtp", 0, CMD_PRM, "Verify ZRTP SAS", verify_sas },
+	{"zrtp", 0, CMD_PRM, "Verify ZRTP SAS <remote ZID>", verify_sas },
 };
 
 
@@ -321,37 +367,48 @@ static int module_init(void)
 	char config_path[256] = "";
 	char zrtp_zid_path[256] = "";
 	FILE *f;
-	int err;
+	int ret, err;
+
+	zrtp_log_set_log_engine(zrtp_log);
 
 	zrtp_config_defaults(&zrtp_config);
-	zrtp_config.cache_type = ZRTP_CACHE_FILE;
+
+	str_ncpy(zrtp_config.client_id, "baresip/zrtp",
+		 sizeof(zrtp_config.client_id));
+
+	zrtp_config.lic_mode = ZRTP_LICENSE_MODE_UNLIMITED;
+
+	zrtp_config.cb.misc_cb.on_send_packet = on_send_packet;
+	zrtp_config.cb.event_cb.on_zrtp_secure = on_zrtp_secure;
 
 	err = conf_path_get(config_path, sizeof(config_path));
 	if (err) {
 		warning("zrtp: could not get config path: %m\n", err);
 		return err;
 	}
-	if (re_snprintf(zrtp_config.cache_file_cfg.cache_path,
-			sizeof(zrtp_config.cache_file_cfg.cache_path),
-			"%s/zrtp_cache.dat", config_path) < 0)
+	ret = re_snprintf(zrtp_config.def_cache_path.buffer,
+			  zrtp_config.def_cache_path.max_length,
+			  "%s/zrtp_cache.dat", config_path);
+	if (ret < 0) {
+		warning("zrtp: could not write cache path\n");
 		return ENOMEM;
+	}
+	zrtp_config.def_cache_path.length = ret;
 
 	if (re_snprintf(zrtp_zid_path,
 			sizeof(zrtp_zid_path),
 			"%s/zrtp_zid", config_path) < 0)
 		return ENOMEM;
 	if ((f = fopen(zrtp_zid_path, "rb")) != NULL) {
-		if (fread(zrtp_config.zid, sizeof(zrtp_config.zid),
-			  1, f) != 1) {
+		if (fread(zid, sizeof(zid), 1, f) != 1) {
 			if (feof(f) || ferror(f)) {
 				warning("zrtp: invalid zrtp_zid file\n");
 			}
 		}
 	}
 	else if ((f = fopen(zrtp_zid_path, "wb")) != NULL) {
-		rand_bytes(zrtp_config.zid, sizeof(zrtp_config.zid));
-		if (fwrite(zrtp_config.zid, sizeof(zrtp_config.zid),
-			   1, f) != 1) {
+		rand_bytes(zid, sizeof(zid));
+		if (fwrite(zid, sizeof(zid), 1, f) != 1) {
 			warning("zrtp: zrtp_zid file write failed\n");
 		}
 		info("zrtp: generated new persistent ZID (%s)\n",
@@ -364,26 +421,18 @@ static int module_init(void)
 	if (f)
 		(void) fclose(f);
 
-	str_ncpy(zrtp_config.client_id, "baresip/zrtp",
-		 sizeof(zrtp_config.client_id));
-	zrtp_config.lic_mode = ZRTP_LICENSE_MODE_UNLIMITED;
-
-	zrtp_config.cb.misc_cb.on_send_packet = on_send_packet;
-	zrtp_config.cb.event_cb.on_zrtp_secure = on_zrtp_secure;
-
 	s = zrtp_init(&zrtp_config, &zrtp_global);
 	if (zrtp_status_ok != s) {
 		warning("zrtp: zrtp_init() failed (status = %d)\n", s);
 		return ENOSYS;
 	}
 
-	menc_register(&menc_zrtp);
+	menc_register(baresip_mencl(), &menc_zrtp);
 
-	debug("zrtp:  cache_file:  %s\n",
-	      zrtp_config.cache_file_cfg.cache_path);
+	debug("zrtp:  cache_file:  %s\n", zrtp_config.def_cache_path.buffer);
 	debug("       zid_file:    %s\n", zrtp_zid_path);
 	debug("       zid:         %w\n",
-	      zrtp_config.zid, sizeof(zrtp_config.zid));
+	      zid, sizeof(zid));
 
 	return cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
 }
