@@ -82,9 +82,10 @@ static struct {
 
 
 /* prototypes */
-static int  ua_call_alloc(struct call **callp, struct ua *ua,
-			  enum vidmode vidmode, const struct sip_msg *msg,
-			  struct call *xcall, const char *local_uri);
+static int ua_call_alloc(struct call **callp, struct ua *ua,
+			 enum vidmode vidmode, const struct sip_msg *msg,
+			 struct call *xcall, const char *local_uri,
+			 bool use_rtp);
 
 
 /* This function is called when all SIP transactions are done */
@@ -197,7 +198,7 @@ int ua_register(struct ua *ua)
 		struct reg *reg = le->data;
 
 		err = reg_register(reg, reg_uri, params,
-				   acc->regint, acc->outbound[i]);
+				   acc->regint, acc->outboundv[i]);
 		if (err) {
 			warning("ua: SIP register failed: %m\n", err);
 			goto out;
@@ -362,7 +363,7 @@ static void call_event_handler(struct call *call, enum call_event ev,
 		ua_printf(ua, "transferring call to %s\n", str);
 
 		err = ua_call_alloc(&call2, ua, VIDMODE_ON, NULL, call,
-				    call_localuri(call));
+				    call_localuri(call), true);
 		if (!err) {
 			struct pl pl;
 
@@ -410,7 +411,8 @@ static void call_dtmf_handler(struct call *call, char key, void *arg)
 
 static int ua_call_alloc(struct call **callp, struct ua *ua,
 			 enum vidmode vidmode, const struct sip_msg *msg,
-			 struct call *xcall, const char *local_uri)
+			 struct call *xcall, const char *local_uri,
+			 bool use_rtp)
 {
 	const struct network *net = baresip_network();
 	struct call_prm cprm;
@@ -437,6 +439,7 @@ static int ua_call_alloc(struct call **callp, struct ua *ua,
 	sa_cpy(&cprm.laddr, net_laddr_af(net, af));
 	cprm.vidmode = vidmode;
 	cprm.af      = af;
+	cprm.use_rtp = use_rtp;
 
 	err = call_alloc(callp, conf_config(), &ua->calls,
 			 ua->acc->dispname,
@@ -459,20 +462,34 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 	struct sip_contact contact;
 	struct call *call = NULL;
 	struct mbuf *desc = NULL;
+	const struct sip_hdr *hdr;
+	bool accept_sdp = true;
 	int err;
 
 	debug("ua: incoming OPTIONS message from %r (%J)\n",
 	      &msg->from.auri, &msg->src);
 
-	err = ua_call_alloc(&call, ua, VIDMODE_ON, NULL, NULL, NULL);
-	if (err) {
-		(void)sip_treply(NULL, uag.sip, msg, 500, "Call Error");
-		return;
+	/* application/sdp is the default if the
+	   Accept header field is not present */
+	hdr = sip_msg_hdr(msg, SIP_HDR_ACCEPT);
+	if (hdr) {
+		accept_sdp = 0==pl_strcasecmp(&hdr->val, "application/sdp");
 	}
 
-	err = call_sdp_get(call, &desc, true);
-	if (err)
-		goto out;
+	if (accept_sdp) {
+
+		err = ua_call_alloc(&call, ua, VIDMODE_ON, NULL, NULL, NULL,
+				    false);
+		if (err) {
+			(void)sip_treply(NULL, uag.sip, msg,
+					 500, "Call Error");
+			return;
+		}
+
+		err = call_sdp_get(call, &desc, true);
+		if (err)
+			goto out;
+	}
 
 	sip_contact_set(&contact, ua_cuser(ua), &msg->dst, msg->tp);
 
@@ -481,16 +498,17 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 			  "Allow: %s\r\n"
 			  "%H"
 			  "%H"
-			  "Content-Type: application/sdp\r\n"
+			  "%s"
 			  "Content-Length: %zu\r\n"
 			  "\r\n"
 			  "%b",
 			  uag_allowed_methods(),
 			  ua_print_supported, ua,
 			  sip_contact_print, &contact,
-			  mbuf_get_left(desc),
-			  mbuf_buf(desc),
-			  mbuf_get_left(desc));
+			  desc ? "Content-Type: application/sdp\r\n" : "",
+			  desc ? mbuf_get_left(desc) : (size_t)0,
+			  desc ? mbuf_buf(desc) : NULL,
+			  desc ? mbuf_get_left(desc) : (size_t)0);
 	if (err) {
 		warning("ua: options: sip_treplyf: %m\n", err);
 	}
@@ -647,9 +665,9 @@ int ua_alloc(struct ua **uap, const char *aor)
 			goto out;
 		}
 
-		for (i=0; i<ARRAY_SIZE(ua->acc->outbound); i++) {
+		for (i=0; i<ARRAY_SIZE(ua->acc->outboundv); i++) {
 
-			if (ua->acc->outbound[i] && ua->acc->regint) {
+			if (ua->acc->outboundv[i] && ua->acc->regint) {
 				err = reg_add(&ua->regl, ua, (int)i+1);
 				if (err)
 					break;
@@ -776,7 +794,7 @@ int ua_connect(struct ua *ua, struct call **callp,
 	if (err)
 		goto out;
 
-	err = ua_call_alloc(&call, ua, vmode, NULL, NULL, from_uri);
+	err = ua_call_alloc(&call, ua, vmode, NULL, NULL, from_uri, true);
 	if (err)
 		goto out;
 
@@ -850,6 +868,21 @@ int ua_answer(struct ua *ua, struct call *call)
 }
 
 
+int ua_progress(struct ua *ua, struct call *call)
+{
+	if (!ua)
+		return EINVAL;
+
+	if (!call) {
+		call = ua_call(ua);
+		if (!call)
+			return ENOENT;
+	}
+
+	return call_progress(call);
+}
+
+
 /**
  * Put the current call on hold and answer the incoming call
  *
@@ -1001,7 +1034,7 @@ void ua_presence_status_set(struct ua *ua, const enum presence_status status)
 const char *ua_outbound(const struct ua *ua)
 {
 	/* NOTE: we pick the first outbound server, should be rotated? */
-	return ua ? ua->acc->outbound[0] : NULL;
+	return ua ? ua->acc->outboundv[0] : NULL;
 }
 
 
@@ -1235,7 +1268,7 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 
 	(void)pl_strcpy(&msg->to.auri, to_uri, sizeof(to_uri));
 
-	err = ua_call_alloc(&call, ua, VIDMODE_ON, msg, NULL, to_uri);
+	err = ua_call_alloc(&call, ua, VIDMODE_ON, msg, NULL, to_uri, true);
 	if (err) {
 		warning("ua: call_alloc: %m\n", err);
 		goto error;
@@ -1537,6 +1570,11 @@ int ua_print_sip_status(struct re_printf *pf, void *unused)
 
 /**
  * Print all calls for a given User-Agent
+ *
+ * @param pf     Print handler for debug output
+ * @param ua     User-Agent
+ *
+ * @return 0 if success, otherwise errorcode
  */
 int ua_print_calls(struct re_printf *pf, const struct ua *ua)
 {
