@@ -255,7 +255,7 @@ static void vidqueue_poll(struct vtx *vtx, uint64_t jfs, uint64_t prev_jfs)
 
 		sent += mbuf_get_left(qent->mb);
 
-		stream_send(vtx->video->strm, qent->marker, qent->pt,
+		stream_send(vtx->video->strm, false, qent->marker, qent->pt,
 			    qent->ts, qent->mb);
 
 		le = le->next;
@@ -654,10 +654,13 @@ static int pt_handler(struct video *v, uint8_t pt_old, uint8_t pt_new)
 
 /* Handle incoming stream data from the network */
 static void stream_recv_handler(const struct rtp_header *hdr,
+				struct rtpext *extv, size_t extc,
 				struct mbuf *mb, void *arg)
 {
 	struct video *v = arg;
 	int err;
+	(void)extv;
+	(void)extc;
 
 	if (!mb)
 		goto out;
@@ -757,7 +760,8 @@ static int vrx_print_pipeline(struct re_printf *pf, const struct vrx *vrx)
 }
 
 
-int video_alloc(struct video **vp, const struct config *cfg,
+int video_alloc(struct video **vp, const struct stream_param *stream_prm,
+		const struct config *cfg,
 		struct call *call, struct sdp_session *sdp_sess, int label,
 		const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		const struct menc *menc, struct menc_sess *menc_sess,
@@ -780,7 +784,8 @@ int video_alloc(struct video **vp, const struct config *cfg,
 	v->cfg = cfg->video;
 	tmr_init(&v->tmr);
 
-	err = stream_alloc(&v->strm, &cfg->avt, call, sdp_sess, "video", label,
+	err = stream_alloc(&v->strm, stream_prm,
+			   &cfg->avt, call, sdp_sess, "video", label,
 			   mnat, mnat_sess, menc, menc_sess,
 			   call_localuri(call),
 			   stream_recv_handler, rtcp_handler, v);
@@ -825,7 +830,7 @@ int video_alloc(struct video **vp, const struct config *cfg,
 	}
 
 	/* Video filters */
-	for (le = list_head(vidfilt_list()); le; le = le->next) {
+	for (le = list_head(baresip_vidfiltl()); le; le = le->next) {
 		struct vidfilt *vf = le->data;
 		void *ctx = NULL;
 
@@ -867,7 +872,8 @@ static int set_vidisp(struct vrx *vrx)
 	vrx->vidisp = mem_deref(vrx->vidisp);
 	vrx->vidisp_prm.view = NULL;
 
-	vd = (struct vidisp *)vidisp_find(vrx->video->cfg.disp_mod);
+	vd = (struct vidisp *)vidisp_find(baresip_vidispl(),
+					  vrx->video->cfg.disp_mod);
 	if (!vd)
 		return ENOENT;
 
@@ -880,7 +886,8 @@ static int set_vidisp(struct vrx *vrx)
 static int set_encoder_format(struct vtx *vtx, const char *src,
 			      const char *dev, struct vidsz *size)
 {
-	struct vidsrc *vs = (struct vidsrc *)vidsrc_find(src);
+	struct vidsrc *vs = (struct vidsrc *)vidsrc_find(baresip_vidsrcl(),
+							 src);
 	int err;
 
 	if (!vs)
@@ -944,7 +951,7 @@ int video_start(struct video *v, const char *peer)
 
 	stream_set_srate(v->strm, SRATE, SRATE);
 
-	if (vidisp_find(NULL)) {
+	if (vidisp_find(baresip_vidispl(), NULL)) {
 		err = set_vidisp(&v->vrx);
 		if (err) {
 			warning("video: could not set vidisp '%s': %m\n",
@@ -955,14 +962,19 @@ int video_start(struct video *v, const char *peer)
 		info("video: no video display\n");
 	}
 
-	size.w = v->cfg.width;
-	size.h = v->cfg.height;
-	err = set_encoder_format(&v->vtx, v->cfg.src_mod,
-				 v->vtx.device, &size);
-	if (err) {
-		warning("video: could not set encoder format to"
-			" [%u x %u] %m\n",
-			size.w, size.h, err);
+	if (vidsrc_find(baresip_vidsrcl(), NULL)) {
+		size.w = v->cfg.width;
+		size.h = v->cfg.height;
+		err = set_encoder_format(&v->vtx, v->cfg.src_mod,
+					 v->vtx.device, &size);
+		if (err) {
+			warning("video: could not set encoder format to"
+				" [%u x %u] %m\n",
+				size.w, size.h, err);
+		}
+	}
+	else {
+		info("video: no video source\n");
 	}
 
 	tmr_start(&v->tmr, TMR_INTERVAL * 1000, tmr_handler, v);
@@ -984,6 +996,8 @@ void video_stop(struct video *v)
 	if (!v)
 		return;
 
+	debug("video: stopping video source ..\n");
+
 	v->started = false;
 	v->vtx.vsrc = mem_deref(v->vtx.vsrc);
 }
@@ -1135,11 +1149,13 @@ int video_decoder_set(struct video *v, struct vidcodec *vc, int pt_rx,
 
 	/* handle vidcodecs without a decoder */
 	if (!vc->decupdh) {
+		struct list *vidcodecl = baresip_vidcodecl();
 		struct vidcodec *vcd;
 
 		info("video: vidcodec '%s' has no decoder\n", vc->name);
 
-		vcd = (struct vidcodec *)vidcodec_find_decoder(vc->name);
+		vcd = (struct vidcodec *)vidcodec_find_decoder(vidcodecl,
+							       vc->name);
 		if (!vcd) {
 			warning("video: could not find decoder (%s)\n",
 				vc->name);
@@ -1279,7 +1295,7 @@ int video_debug(struct re_printf *pf, const struct video *v)
 	err |= re_hprintf(pf, "     n_intra=%u, n_picup=%u\n",
 			  vrx->n_intra, vrx->n_picup);
 
-	if (!list_isempty(vidfilt_list())) {
+	if (!list_isempty(baresip_vidfiltl())) {
 		err |= vtx_print_pipeline(pf, vtx);
 		err |= vrx_print_pipeline(pf, vrx);
 	}
@@ -1301,7 +1317,8 @@ int video_print(struct re_printf *pf, const struct video *v)
 
 int video_set_source(struct video *v, const char *name, const char *dev)
 {
-	struct vidsrc *vs = (struct vidsrc *)vidsrc_find(name);
+	struct vidsrc *vs = (struct vidsrc *)vidsrc_find(baresip_vidsrcl(),
+							 name);
 	struct vtx *vtx;
 
 	if (!v)
